/*
 * Copyright 2012 astamuse company,Ltd.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */

package com.astamuse.asta4d.render.transformer;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

import org.jsoup.nodes.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.astamuse.asta4d.extnode.GroupNode;

public class FutureTransformer extends Transformer<Future<?>> {

    private final static Logger logger = LoggerFactory.getLogger(FutureTransformer.class);

    public FutureTransformer(Future<?> content) {
        super(content);
    }

    @Override
    protected Element transform(Element elem, Future<?> future) {
        try {
            Object result = future.get();
            Transformer<?> transformer = TransformerFactory.generateTransformer(result);
            return transformer.invoke(elem);
        } catch (InterruptedException | ExecutionException e) {
            logger.error("Failed retrieve result from future:" + e.getMessage(), e);
            return new GroupNode();
        }
    }

}
